@@ -1,4 +1,4 @@
-# Copyright 2015-2017 Google Inc. All Rights Reserved.
+# Copyright 2015 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -192,7 +192,7 @@ class _SplitPenaltyAssigner(pytree_visitor.PyTreeVisitor):
         _SetStronglyConnected(node.children[1])
         if (len(node.children[1].children) > 1 and
             pytree_utils.NodeName(node.children[1].children[1]) == 'comp_for'):
-          # Don't penalize spliting before a comp_for expression.
+          # Don't penalize splitting before a comp_for expression.
           _SetSplitPenalty(_FirstChildNode(node.children[1]), 0)
         else:
           _SetSplitPenalty(
@@ -421,7 +421,7 @@ class _SplitPenaltyAssigner(pytree_visitor.PyTreeVisitor):
     if node.children[0].value == '(':
       if node.children[-1].value == ')':
         if pytree_utils.NodeName(node.parent) == 'if_stmt':
-          _SetSplitPenalty(node.children[-1], UNBREAKABLE)
+          _SetSplitPenalty(node.children[-1], STRONGLY_CONNECTED)
         else:
           if len(node.children) > 2:
             _SetSplitPenalty(_FirstChildNode(node.children[1]), EXPR)
