@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2015-2016 Google Inc. All Rights Reserved.
+# Copyright 2015 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,10 +14,8 @@
 # limitations under the License.
 """Tests for yapf.file_resources."""
 
-import contextlib
 import os
 import shutil
-import sys
 import tempfile
 import unittest
 
@@ -25,15 +23,7 @@ from yapf.yapflib import errors
 from yapf.yapflib import file_resources
 from yapf.yapflib import py3compat
 
-
-@contextlib.contextmanager
-def stdout_redirector(stream):  # pylint: disable=invalid-name
-  old_stdout = sys.stdout
-  sys.stdout = stream
-  try:
-    yield
-  finally:
-    sys.stdout = old_stdout
+from yapftests import utils
 
 
 class GetDefaultStyleForDirTest(unittest.TestCase):
@@ -72,12 +62,14 @@ class GetCommandLineFilesTest(unittest.TestCase):
 
   def setUp(self):
     self.test_tmpdir = tempfile.mkdtemp()
+    self.old_dir = os.getcwd()
 
   def tearDown(self):
     shutil.rmtree(self.test_tmpdir)
+    os.chdir(self.old_dir)
 
   def _make_test_dir(self, name):
-    fullpath = os.path.join(self.test_tmpdir, name)
+    fullpath = os.path.normpath(os.path.join(self.test_tmpdir, name))
     os.makedirs(fullpath)
     return fullpath
 
@@ -115,7 +107,8 @@ class GetCommandLineFilesTest(unittest.TestCase):
     tdir3 = self._make_test_dir('test3/foo/bar/bas/xxx')
     files = [
         os.path.join(tdir1, 'testfile1.py'),
-        os.path.join(tdir2, 'testfile2.py'), os.path.join(tdir3, 'testfile3.py')
+        os.path.join(tdir2, 'testfile2.py'),
+        os.path.join(tdir3, 'testfile3.py'),
     ]
     _touch_files(files)
 
@@ -131,7 +124,8 @@ class GetCommandLineFilesTest(unittest.TestCase):
     tdir3 = self._make_test_dir('test3/foo/bar/bas/xxx')
     files = [
         os.path.join(tdir1, 'testfile1.py'),
-        os.path.join(tdir2, 'testfile2.py'), os.path.join(tdir3, 'testfile3.py')
+        os.path.join(tdir2, 'testfile2.py'),
+        os.path.join(tdir3, 'testfile3.py'),
     ]
     _touch_files(files)
 
@@ -141,9 +135,46 @@ class GetCommandLineFilesTest(unittest.TestCase):
                 [self.test_tmpdir], recursive=True, exclude=['*test*3.py'])),
         sorted([
             os.path.join(tdir1, 'testfile1.py'),
-            os.path.join(tdir2, 'testfile2.py')
+            os.path.join(tdir2, 'testfile2.py'),
         ]))
 
+  def test_find_with_excluded_dirs(self):
+    tdir1 = self._make_test_dir('test1')
+    tdir2 = self._make_test_dir('test2/testinner/')
+    tdir3 = self._make_test_dir('test3/foo/bar/bas/xxx')
+    files = [
+        os.path.join(tdir1, 'testfile1.py'),
+        os.path.join(tdir2, 'testfile2.py'),
+        os.path.join(tdir3, 'testfile3.py'),
+    ]
+    _touch_files(files)
+
+    os.chdir(self.test_tmpdir)
+
+    found = sorted(
+        file_resources.GetCommandLineFiles(
+            ['test1', 'test2', 'test3'],
+            recursive=True,
+            exclude=[
+                'test1',
+                'test2/testinner/',
+            ]))
+
+    self.assertEqual(found, ['test3/foo/bar/bas/xxx/testfile3.py'])
+
+    found = sorted(
+        file_resources.GetCommandLineFiles(
+            ['.'], recursive=True, exclude=[
+                'test1',
+                'test3',
+            ]))
+
+    self.assertEqual(found, ['./test2/testinner/testfile2.py'])
+
+  def test_find_with_excluded_current_dir(self):
+    with self.assertRaises(errors.YapfError):
+      file_resources.GetCommandLineFiles([], False, exclude=['./z'])
+
 
 class IsPythonFileTest(unittest.TestCase):
 
@@ -188,6 +219,22 @@ class IsPythonFileTest(unittest.TestCase):
     self.assertFalse(file_resources.IsPythonFile(file1))
 
 
+class IsIgnoredTest(unittest.TestCase):
+
+  def test_root_path(self):
+    self.assertTrue(file_resources.IsIgnored('media', ['media']))
+    self.assertFalse(file_resources.IsIgnored('media', ['media/*']))
+
+  def test_sub_path(self):
+    self.assertTrue(file_resources.IsIgnored('media/a', ['*/a']))
+    self.assertTrue(file_resources.IsIgnored('media/b', ['media/*']))
+    self.assertTrue(file_resources.IsIgnored('media/b/c', ['*/*/c']))
+
+  def test_trailing_slash(self):
+    self.assertTrue(file_resources.IsIgnored('z', ['z']))
+    self.assertTrue(file_resources.IsIgnored('z', ['z/']))
+
+
 class BufferedByteStream(object):
 
   def __init__(self):
@@ -212,19 +259,19 @@ class WriteReformattedCodeTest(unittest.TestCase):
     shutil.rmtree(cls.test_tmpdir)
 
   def test_write_to_file(self):
-    s = u'foobar'
-    with tempfile.NamedTemporaryFile(dir=self.test_tmpdir) as testfile:
+    s = u'foobar\n'
+    with utils.NamedTempFile(dirname=self.test_tmpdir) as (f, fname):
       file_resources.WriteReformattedCode(
-          testfile.name, s, in_place=True, encoding='utf-8')
-      testfile.flush()
+          fname, s, in_place=True, encoding='utf-8')
+      f.flush()
 
-      with open(testfile.name) as f:
-        self.assertEqual(f.read(), s)
+      with open(fname) as f2:
+        self.assertEqual(f2.read(), s)
 
   def test_write_to_stdout(self):
     s = u'foobar'
     stream = BufferedByteStream() if py3compat.PY3 else py3compat.StringIO()
-    with stdout_redirector(stream):
+    with utils.stdout_redirector(stream):
       file_resources.WriteReformattedCode(
           None, s, in_place=False, encoding='utf-8')
     self.assertEqual(stream.getvalue(), s)
@@ -232,7 +279,7 @@ class WriteReformattedCodeTest(unittest.TestCase):
   def test_write_encoded_to_stdout(self):
     s = '\ufeff# -*- coding: utf-8 -*-\nresult = "passed"\n'  # pylint: disable=anomalous-unicode-escape-in-string
     stream = BufferedByteStream() if py3compat.PY3 else py3compat.StringIO()
-    with stdout_redirector(stream):
+    with utils.stdout_redirector(stream):
       file_resources.WriteReformattedCode(
           None, s, in_place=False, encoding='utf-8')
     self.assertEqual(stream.getvalue(), s)
